﻿--[[---------------------------------------------------------------------------

Copyright (c) 2008 by K. Scott Piel 
All Rights Reserved

E-mail: < kscottpiel@gmail.com >
Web:    < http://www.scottpiel.com >

This file is part of nUI.

    nUI is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    nUI is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with nUI.  If not, see <http://www.gnu.org/licenses/>.
	
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE 
USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

--]]---------------------------------------------------------------------------
	
if not nUI then nUI = {}; end
if not nUI_Options then nUI_Options = {}; end
if not nUI_DefaultConfig then nUI_DefaultConfig = {}; end

nUI_Options.minimap = true;

nUI_DefaultConfig.Minimap =
{
	enabled = true,
	
	anchor = 
	{
		anchor_pt   = "CENTER",
		relative_to = nUI_Dashboard,
		relative_pt = "CENTER",
		xOfs        = 0,
		yOfs        = -68,
	},
	
	options =
	{
		enabled  = true;
		height   = 225;
		width    = 225;
		strata   = nil,
		level    = 2,
	},
};

-------------------------------------------------------------------------------
-- master frame for minimap management

local background = CreateFrame( "Frame", "nUI_MinimapManagerBackground", nUI_Dashboard.Anchor );
local frame      = CreateFrame( "Frame", "nUI_MinimapManager", nUI_Dashboard.Anchor, "SecureHandlerStateTemplate" );

background:SetAllPoints( frame );

-------------------------------------------------------------------------------
-- event management

local function onMinimapEvent()
	
	if event == "VARIABLES_LOADED" then

		nUI:registerSkinnedFrame( frame );
		nUI:registerScalableFrame( frame );
	
		-- set up a slash command handler for choosing the minimap shape
		
		local option = nUI_SlashCommands[nUI_SLASHCMD_ROUNDMAP];
		
		nUI_SlashCommands:setHandler( option.command,
			
			function( msg, arg1 )
				
				nUI_Options.round_map = not nUI_Options.round_map;
				
				DEFAULT_CHAT_FRAME:AddMessage( (option.message):format( nUI_Options.round_map and nUI_L["round"] or nUI_L["square"] ), 1, 0.83, 0 );

				if nUI_Options.round_map then
					Minimap:SetMaskTexture( "Interface\\AddOns\\nUI\\Integration\\InfoPanels\\Art\\nUI_RoundMinimapMask" );
				else
					Minimap:SetMaskTexture( "Interface\\AddOns\\nUI\\Integration\\InfoPanels\\Art\\nUI_SquareMinimapMask" );
				end
			end
		);				
	
		-- set up a slash command handler for choosing whether or not nUI will manage the minimap
		
		local option = nUI_SlashCommands[nUI_SLASHCMD_MINIMAP];
		
		nUI_SlashCommands:setHandler( option.command,
			
			function( msg, arg1 )
				
				nUI_Options.minimap = not nUI_Options.minimap;
				
				DEFAULT_CHAT_FRAME:AddMessage( (option.message):format( nUI_Options.minimap and nUI_L["|cFF00FF00ENABLED|r"] or nUI_L["|cFFFF0000DISABLED|r"] ), 1, 0.83, 0 );

				ReloadUI();

			end
		);		
	
		-- let the user know if we're managing the minimap or not
		
		DEFAULT_CHAT_FRAME:AddMessage( (option.message):format( nUI_Options.minimap and nUI_L["|cFF00FF00ENABLED|r"] or nUI_L["|cFFFF0000DISABLED|r"] ), 1, 0.83, 0 );
	
	-- lay out the minimap on the dashboard
	
	elseif event == "PLAYER_ENTERING_WORLD" then
	
		-- create the show/hide logic for the minimap frame... if we have a focus frame, then we hide the minimap
		-- frame, otherwise, the minimap frame is visible
		
		frame:SetAttribute( "unit", "focus" );

		frame:SetAttribute( "_onstate-unitexists",
			[[
				if UnitExists( "focus" )
				or UnitExists( "vehicle" )
				then self:Hide();
				else self:Show();
				end
			]]
		);
		
		-- we only need to do this once
		
		frame:UnregisterEvent( "PLAYER_ENTERING_WORLD" );
		
	end		
end

frame:SetScript( "OnEvent", onMinimapEvent );
frame:RegisterEvent( "VARIABLES_LOADED" );
frame:RegisterEvent( "PLAYER_ENTERING_WORLD" );

-------------------------------------------------------------------------------

frame.applyScale = function( scale )
	
	local anchor   = scale and frame.anchor or nil;
	local scale    = scale or frame.scale or 1;
	local options  = frame.options;
	
	if options then
	
		local height   = options.height * scale * nUI.scale;
		local width    = options.width * scale * nUI.scale;
		
		if frame.width    ~= width
		or frame.height   ~= height
		then
			
			frame.height   = height;
			frame.width    = width;
			
			frame:SetHeight( height );
			frame:SetWidth( width );
				
			local xOfs     = MinimapCluster:GetLeft() - Minimap:GetLeft();
			local yOfs     = MinimapCluster:GetBottom() - Minimap:GetBottom();
			
			nUI_Movers:lockFrame( MinimapCluster, false, nil );
			
			MinimapCluster:ClearAllPoints();
			MinimapCluster:SetPoint( "BOTTOMLEFT", frame, "BOTTOMLEFT", xOfs, yOfs+2);
			
			MinimapCluster:SetScale( width / Minimap:GetWidth() );
			
			nUI_Movers:lockFrame( MinimapCluster, true, nil );
			
		end
	end
end

-------------------------------------------------------------------------------

frame.applyAnchor = function( anchor )
	
	local anchor = anchor or frame.anchor or {};
	local anchor_pt   = anchor.anchor_pt or "CENTER";
	local relative_to = anchor.relative_to or frame:GetParent():GetName();
	local relative_pt = anchor.relative_pt or anchor_pt;
	local xOfs        = (anchor.xOfs or 0) * nUI.scale;
	local yOfs        = (anchor.yOfs or 0) * nUI.scale;
	
	frame.anchor = anchor;
	
	if frame.xOfs ~= xOfs
	or frame.yOfs ~= yOfs
	or frame.anchor_pt ~= anchor_pt
	or frame.relative_to ~= relative_to
	or frame.relative_pt ~= relative_pt
	then
		
		frame.anchor_pt   = anchor_pt;
		frame.relative_to = relative_to;
		frame.relative_pt = relative_pt;
		frame.xOfs        = xOfs;
		frame.yOfs        = yOfs;
		
		frame:ClearAllPoints();
		frame:SetPoint( anchor_pt, relative_to, relative_pt, xOfs, yOfs );
		
	end
end

-------------------------------------------------------------------------------

frame.applyOptions = function( options )
	
	frame.options = options;
	
	if not options 
	or not options.enabled 
	or not nUI_Options.minimap
	then
		
--		nUI:debug( "nUI_Minimap: minimap manager is disabled", 1 );
		
		frame:SetAlpha( 0 );
		
	else
	
--		nUI:debug( "nUI_Minimap: minimap manager is enabled", 1 );

		if UnitExists( "focus" ) 
		or UnitExists( "vehicle" )
		then frame:Hide();
		else frame:Show();
		end
		
		background:SetFrameStrata( options.strata or frame:GetParent():GetFrameStrata() );
		background:SetFrameLevel( frame:GetParent():GetFrameLevel() + (options.level or 1) );
		
		frame:SetFrameStrata( background:GetFrameStrata() );
		frame:SetFrameLevel( background:GetFrameLevel()+1 );
		frame:SetAlpha( 1 );
		
		-- if there's a border, set it
		
		if options.border then
				
			local border_color = options.border.color.border;
			local backdrop_color = options.border.color.backdrop;
			
			frame:SetBackdrop( options.border.backdrop );
			frame:SetBackdropBorderColor( border_color.r, border_color.g, border_color.b, border_color.a );
			frame:SetBackdropColor( backdrop_color.r, backdrop_color.g, backdrop_color.b, backdrop_color.a );
	
		else 
			
			frame:SetBackdrop( nil );
			
		end
		
		-- if there's a background, set it
		
		if options.background then
				
			local border_color = options.background.color.border;
			local backdrop_color = options.background.color.backdrop;
			
			background:SetBackdrop( options.background.backdrop );
			background:SetBackdropBorderColor( border_color.r, border_color.g, border_color.b, border_color.a );
			background:SetBackdropColor( backdrop_color.r, backdrop_color.g, backdrop_color.b, backdrop_color.a );
	
		else 
			
			background:SetBackdrop( nil );
			
		end

		-- okay -- now attach the minimap to the frame
		
		MinimapCluster:EnableMouse( false );
		MinimapCluster:SetParent( frame );
		MinimapCluster:SetFrameStrata( frame:GetFrameStrata() );
		MinimapCluster:SetFrameLevel( frame:GetFrameLevel()+1 );
	
		MinimapCluster:SetMovable( true );
		MinimapCluster:StartMoving()
		MinimapCluster:StopMovingOrSizing()
		
		nUI_Movers:lockFrame( MinimapCluster, true, nil );
	
		MinimapBorder:SetParent( nUI.BlizUI );
		MinimapBorderTop:SetParent( nUI.BlizUI );
		MinimapToggleButton:SetParent( nUI.BlizUI );
		MinimapZoneTextButton:SetParent( nUI.BlizUI );
						
		MiniMapMailFrame:SetParent( Minimap );
		MiniMapMailFrame:SetScale( 0.7 );
		MiniMapMailFrame:ClearAllPoints();
		MiniMapMailFrame:RegisterForDrag();
		MiniMapMailFrame:SetScript( "OnDragStart", nil );
		MiniMapMailFrame:SetScript( "OnDragStop", nil );
		MiniMapMailFrame:SetPoint( "BOTTOMLEFT", MiniMapTracking, "TOPRIGHT", -2, -10 );
		MiniMapMailFrame:SetFrameStrata( Minimap:GetFrameStrata() );
		MiniMapMailFrame:SetFrameLevel( Minimap:GetFrameLevel()+1 );
		
		MiniMapBattlefieldFrame:SetParent( Minimap );
		MiniMapBattlefieldFrame:SetScale( 0.8 );
		MiniMapBattlefieldFrame:ClearAllPoints();
		MiniMapBattlefieldFrame:RegisterForDrag();
		MiniMapBattlefieldFrame:SetScript( "OnDragStart", nil );
		MiniMapBattlefieldFrame:SetScript( "OnDragStop", nil );
		MiniMapBattlefieldFrame:SetPoint( "BOTTOMRIGHT", MiniMapWorldMapButton, "TOPLEFT", 2, -10 );
		MiniMapBattlefieldFrame:SetFrameStrata( Minimap:GetFrameStrata() );
		MiniMapBattlefieldFrame:SetFrameLevel( Minimap:GetFrameLevel()+1 );
		
		MiniMapWorldMapButton:SetParent( Minimap );
		MiniMapWorldMapButton:SetScale( 0.8 );
		MiniMapWorldMapButton:ClearAllPoints();
		MiniMapWorldMapButton:RegisterForDrag();
		MiniMapWorldMapButton:SetScript( "OnDragStart", nil );
		MiniMapWorldMapButton:SetScript( "OnDragStop", nil );
		MiniMapWorldMapButton:SetPoint( "TOPRIGHT", Minimap, "TOPRIGHT", 2, 1 );
		MiniMapWorldMapButton:SetFrameStrata( Minimap:GetFrameStrata() );
		MiniMapWorldMapButton:SetFrameLevel( Minimap:GetFrameLevel()+1 );
		
		MiniMapTracking:SetParent( Minimap );
		MiniMapTracking:SetScale( 0.65 );
		MiniMapTracking:ClearAllPoints();
		MiniMapTracking:RegisterForDrag();
		MiniMapTracking:SetScript( "OnDragStart", nil );
		MiniMapTracking:SetScript( "OnDragStop", nil );
		MiniMapTracking:SetPoint( "TOPLEFT", Minimap, "TOPLEFT", 0, 1 );
		MiniMapTracking:SetFrameStrata( Minimap:GetFrameStrata() );
		MiniMapTracking:SetFrameLevel( Minimap:GetFrameLevel()+1 );

		MinimapZoomIn:SetParent( Minimap );
		MinimapZoomIn:SetScale( 0.8 );
		MinimapZoomIn:RegisterForDrag();
		MinimapZoomIn:ClearAllPoints();
		MinimapZoomIn:SetScript( "OnDragStart", nil );
		MinimapZoomIn:SetScript( "OnDragStop", nil );
		MinimapZoomIn:SetPoint( "BOTTOMRIGHT", Minimap, "BOTTOMRIGHT", 1, -1 );
		MinimapZoomIn:SetFrameStrata( Minimap:GetFrameStrata() );
		MinimapZoomIn:SetFrameLevel( Minimap:GetFrameLevel()+1 );
	
		MinimapZoomOut:SetParent( Minimap );
		MinimapZoomOut:SetScale( 0.8 );
		MinimapZoomOut:ClearAllPoints();
		MinimapZoomOut:RegisterForDrag();
		MinimapZoomOut:SetScript( "OnDragStart", nil );
		MinimapZoomOut:SetScript( "OnDragStop", nil );
		MinimapZoomOut:SetPoint( "BOTTOMLEFT", Minimap, "BOTTOMLEFT", -1, -1 );
		MinimapZoomOut:SetFrameStrata( Minimap:GetFrameStrata() );
		MinimapZoomOut:SetFrameLevel( Minimap:GetFrameLevel()+1 );				

		MiniMapCompassRing:ClearAllPoints();
		MiniMapCompassRing:SetParent( Minimap );
		MiniMapCompassRing:SetPoint( "CENTER", Minimap, "CENTER", 0, 0 );
				
		Minimap:EnableMouseWheel( true );	
		
		if nUI_Options.round_map then
			Minimap:SetMaskTexture( "Interface\\AddOns\\nUI\\Integration\\InfoPanels\\Art\\nUI_RoundMinimapMask" );
		else
			Minimap:SetMaskTexture( "Interface\\AddOns\\nUI\\Integration\\InfoPanels\\Art\\nUI_SquareMinimapMask" );
		end
		
		Minimap:SetScript( "OnMouseWheel", function()
			if arg1 > 0 then Minimap_ZoomIn();
			else Minimap_ZoomOut();
			end	
		end	);
			
		-- and size it all
		
		frame.applyScale( options.scale or frame.scale or 1 );
	end
end

-------------------------------------------------------------------------------

frame.applySkin = function( skin )
	
	local skin = skin and skin.Minimap or nUI_DefaultConfig.Minimap;
	
	if skin and skin.enabled then
		
--		nUI:debug( "nUI_Minimap: setting skin", 1 );
		
		frame.applyOptions( skin.options );
		frame.applyAnchor( skin.anchor );
		RegisterUnitWatch( frame, true );
	
	else
		
		UnregisterUnitWatch( frame );
		frame:SetScript( "OnUpdate", nil );
		frame:SetAlpha( 0 );
		
	end
end